﻿Type=Service
Version=1.70
StartAtBoot=False
@EndOfDesignText@
'Service module
Sub Process_Globals
	Dim Server As ServerSocket
	Dim Client As Socket
	Dim Streams As AsyncStreams
	Dim Notification1 As Notification
	Dim PE As PhoneEvents
	Dim Phone As Phone
	Dim PhoneId As PhoneId
	Dim LogCat As LogCat
	Dim BC As ByteConverter
	Dim ConnectedStatus As Boolean
	Dim Out As OutputStream
	Dim PING, APK_COPY_START, APK_PACKET, APK_COPY_CLOSE, SEND_DEVICE_NAME As Byte
	Dim START_LOGCAT, STOP_LOGCAT, LOGCAT_DATA As Byte
	Dim LAUNCH_PROGRAM, KILL_PROGRAM, ASK_FOR_DESIGNER As Byte
	PING = 1
	APK_COPY_START = 2
	APK_PACKET = 3
	APK_COPY_CLOSE = 4
	SEND_DEVICE_NAME = 5
	START_LOGCAT = 6
	STOP_LOGCAT = 7
	LOGCAT_DATA = 8
	LAUNCH_PROGRAM = 9
	KILL_PROGRAM = 10
	ASK_FOR_DESIGNER = 11
	Dim DisconnectTimer As Timer
	Dim DisconnectTicks As Int
	
	' ADDED
	Dim SEND_SDROOT, SEND_DIRSANDFILES, SEND_DIRS, SEND_FILES, END_DIRSANDFILES As Byte
	SEND_SDROOT = 100
	SEND_DIRSANDFILES  = 101 ' given directory
	SEND_DIRS  = 102 
	SEND_FILES  = 103 
	END_DIRSANDFILES  = 104 ' given directory
	
	Dim FILE_COPY_START, FILE_PACKET, FILE_COPY_CLOSE As Byte
	FILE_COPY_START = 110 ' given directory and filename
	FILE_PACKET = 111
	FILE_COPY_CLOSE = 112
	
	Dim FILE_SEND, FILE_DELETE As Byte
	FILE_SEND = 120 ' given directory and filename
	FILE_DELETE = 121 ' given diectory and filename	
	' /ADDED
End Sub

Sub Service_Create
	Server.Initialize(6789, "Server")
	DisconnectTimer.Initialize("DisconnectTimer", 1000)
	Notification1.Initialize
	Notification1.Icon = "icon"
	Notification1.Light = False
	Notification1.Sound = False
	Notification1.Vibrate = False
	Notification1.SetInfo("B4A-Bridge", "Disconnected", Main)
	Service.StartForeground(1, Notification1)
	PE.Initialize("PE")
End Sub

Sub Service_Start
	UpdateStatus (ConnectedStatus, True)
End Sub

Sub UpdateStatus (Connected As Boolean, AllowFurtherConnections As Boolean)
	ConnectedStatus = Connected
	If Connected Then
		DisconnectTimer.Enabled = True
		DisconnectTicks = 0
		Notification1.SetInfo("B4A-Bridge", "Connected", Main)
		CheckIfDesignerIsInstalled
	Else
		If Streams.IsInitialized Then Streams.Close
		DisconnectTimer.Enabled = False
		Notification1.SetInfo("B4A-Bridge", "Disconnected", Main)
	End If
	If AllowFurtherConnections Then
		'Service should already be initialized here.
		Notification1.Notify(1)
		Server.Listen
	Else
		Server.Close
	End If
	CallSub(Main, "UpdateStatus")
End Sub

Sub DisconnectTimer_Tick
	DisconnectTicks = DisconnectTicks + 1
	If DisconnectTicks = 10 Then
		UpdateStatus(False, True)
	End If
End Sub

Sub Service_Destroy
	Service.StopForeground(1)
	UpdateStatus (False, False)
	LogCat.LogCatStop
End Sub

Sub Server_NewConnection (Successful As Boolean, NewSocket As Socket)
	If Streams.IsInitialized Then Streams.Close
	If Successful Then
		Log("Connected to B4A-Bridge")
		Client = NewSocket
		Streams.InitializePrefix(Client.InputStream, False, Client.OutputStream, _
			"Streams")
		Dim pingB(1) As Byte
		pingB(0) = PING
		Streams.Write(pingB)
		Dim name As String
		name = Phone.Manufacturer & " " & Phone.Model & "-" & PhoneId.GetDeviceId
		Dim nameBytes() As Byte
		nameBytes = name.GetBytes("UTF8")
		Streams.Write(AddCommandToBytes(SEND_DEVICE_NAME, nameBytes, nameBytes.Length))
		UpdateStatus(True, True)
	Else
		ToastMessageShow(LastException.Message, True)
		Log("Error connecting: " & LastException.Message)
		UpdateStatus(False, True)
	End If
End Sub

Sub Streams_Error
	ToastMessageShow(LastException.Message, True)
	UpdateStatus(False, True)
	Log("Streams_Error: " & LastException.Message)
End Sub

Sub Streams_Terminated
	UpdateStatus(False, True)
	Log("Streams_terminated")
End Sub



Sub AddCommandToBytes(Command As Byte, Buffer() As Byte, Length As Int) As Byte()
	Dim b(Length + 1) As Byte
	b(0) = Command
	BC.ArrayCopy(Buffer, 0, b, 1, Length)
	Return b
End Sub

Sub Streams_NewData (Buffer() As Byte)
	If Streams.IsInitialized = False Then Return
	DisconnectTicks = 0 'reset the disconnect timer
	Dim command As Byte
	command = Buffer(0)
	Select command
		Case PING
			Dim b(1) As Byte
			b(0) = PING
			Streams.Write(b)
		Case APK_COPY_START
			HandleFileStart
		Case APK_PACKET
			HandleFilePacket(Buffer)
		Case APK_COPY_CLOSE
			HandleFileClose
		Case START_LOGCAT
			StartLogcat(Buffer)
		Case STOP_LOGCAT
			StopLogcat
		Case LAUNCH_PROGRAM
			LaunchProgram(Buffer)
		Case KILL_PROGRAM
			KillProgram(Buffer)
		' ADDED
		Case SEND_SDROOT
			xSendSDRoot
		Case SEND_DIRSANDFILES
			xSendDirsAndFiles(buffer)	
		Case FILE_COPY_START
			xHandleCopyStart(buffer)
		Case FILE_PACKET
			xHandleCopyPacket(Buffer)
		Case FILE_COPY_CLOSE
			xHandleCopyClose			
		Case FILE_SEND
			xHandleFileSend(buffer)			
		Case FILE_DELETE
			xHandleFileDelete(buffer)			
		' /ADDED
	End Select
End Sub

Sub CheckIfDesignerIsInstalled
	Dim pm As PackageManager
	Dim packages As List
	packages = pm.GetInstalledPackages
	Dim buffer() As Byte
	If packages.IndexOf("anywheresoftware.b4a.designer") = -1 Then
		buffer = "0".GetBytes("UTF8")
	Else
		Dim s As String
		s = pm.GetVersionCode("anywheresoftware.b4a.designer")
		buffer = s.GetBytes("UTF8")
	End If
	
	'we are sending the installed designer version so the IDE will decide if a new version should be installed.
	Streams.Write(AddCommandToBytes(ASK_FOR_DESIGNER, buffer, buffer.Length))
End Sub

Sub LaunchProgram (Buffer() As Byte)
	Dim s As String
	s = BytesToString(Buffer, 1, Buffer.Length - 1, "UTF8")
	Phone.Shell("am", Regex.Split(",", s), Null, Null)
End Sub

Sub KillProgram (Buffer() As Byte)
	Dim package As String
	package = BytesToString(Buffer, 1, Buffer.Length - 1, "UTF8")
	Dim sb As StringBuilder
	sb.Initialize
	Phone.Shell("ps", Null, sb, Null)
	Dim m As Matcher
	m = Regex.Matcher2("^[^ ]*\s+(\d+) .*" & package, Regex.MULTILINE, sb.ToString)
	If m.Find Then
		Phone.Shell("kill", Array As String(m.Group(1)), Null, Null)
	End If
End Sub

Sub StartLogcat (Buffer() As Byte)
	Dim args As String
	args = BytesToString(Buffer, 1, Buffer.Length - 1, "UTF8")
	LogCat.LogCatStart(Regex.Split(",", args), "LogCat")
End Sub

Sub LogCat_LogCatData (Buffer() As Byte, Length As Int)
	If ConnectedStatus = True Then
		Streams.Write(AddCommandToBytes(LOGCAT_DATA, Buffer, Length))
	End If
End Sub

Sub StopLogcat
	LogCat.LogCatStop
End Sub

Sub HandleFileStart
	Log("Installing file.")
	If out.IsInitialized Then out.Close
	Out = File.OpenOutput(Main.folder, "temp.apk", False)
End Sub

Sub HandleFilePacket(Buffer() As Byte)
	Out.WriteBytes(Buffer, 1, Buffer.Length - 1)
End Sub

Sub HandleFileClose
	Out.Close
	'ask the system to install the apk
	Dim i As Intent
	i.Initialize(i.ACTION_VIEW, "file://" & File.Combine(Main.folder, "temp.apk"))
	i.SetType("application/vnd.android.package-archive")
	StartActivity(i)
End Sub

Sub PE_PackageAdded (Package As String, Intent As Intent)
	Log("PackageAdded: " & Package)
End Sub

Sub PE_ConnectivityChanged (NetworkType As String, State As String, Intent As Intent)
	If NetworkType = "WIFI" Then CallSub(Main, "UpdateIp")
End Sub

' ADDED
Sub xSendSDRoot
	Dim bytes() As Byte
	bytes = File.DirRootExternal.GetBytes("UTF8")
	Streams.Write(AddCommandToBytes(SEND_SDROOT, bytes, bytes.Length))
End Sub

Sub xSendDirsAndFiles(buffer() As Byte)
	Dim bytes() As Byte
	Dim i As Double
	Dim dir, fileinfo As String
	Dim dirs, files, dirsandfiles As List
	dirs.Initialize
	files.Initialize
	dir = BytesToString(buffer, 1, buffer.Length - 1, "UTF-8")
	Log("Sending dirs and files " & dir)
	dirsandfiles = File.ListFiles(dir)
	If Not(dirsandfiles.IsInitialized) Then ' unitialised list returned for inaccessible folder
		dirsandfiles.Initialize
	End If
	Dim filedata(dirsandfiles.Size * 2) As String
	For i = 0 To dirsandfiles.Size - 1
		fileinfo = dirsandfiles.Get(i)
		If File.IsDirectory(dir, dirsandfiles.Get(i)) Then
			fileinfo = fileinfo & ",Dir"
			dirs.Add(fileinfo)
		Else
			fileinfo = fileinfo & "," & File.Size(dir, dirsandfiles.Get(i))
			files.Add(fileinfo)
		End If
	Next
	dirs.Sort(True)
	files.sort(True)
	For i = 0 To dirs.Size - 1
		fileinfo = dirs.Get(i)
		bytes = fileinfo.GetBytes("UTF8")		
		Streams.Write(AddCommandToBytes(SEND_DIRS, bytes, bytes.Length))	
	Next
	For i = 0 To files.Size - 1
		fileinfo = files.Get(i)
		bytes = fileinfo.GetBytes("UTF8")		
		Streams.Write(AddCommandToBytes(SEND_FILES, bytes, bytes.Length))	
	Next
		Dim bytes(1) As Byte
		bytes(0) = END_DIRSANDFILES
		Streams.Write(bytes)	
End Sub

Sub xHandleCopyStart(buffer() As Byte)
	Dim bytes() As Byte
	Dim data(0) As String
	Dim filename As String
	filename = BytesToString(buffer, 1, buffer.Length - 1, "UTF-8")
	data = Regex.Split(",", filename)
	Log("Copying file "  & filename)
	If out.IsInitialized Then out.Close
	Out = File.OpenOutput(data(0), data(1), False)
End Sub

Sub xHandleCopyPacket(Buffer() As Byte)
	Out.WriteBytes(Buffer, 1, Buffer.Length - 1)
End Sub

Sub xHandleCopyClose
	Out.Close
End Sub

Sub	xHandleFileSend(buffer() As Byte)	
	Dim bytes() As Byte
	Dim data(0) As String
	Dim filename As String
	Dim numb, numbytes, packets As Long
	filename = BytesToString(buffer, 1, buffer.Length - 1, "UTF-8")
	data = Regex.Split(",", filename)
	Log("Sending file " & filename)
	Dim in As InputStream
	in = File.OpenInput(data(0), data(1))	
	Dim Temp(4096) As Byte
	Do Until numb = -1
		numb = in.ReadBytes(Temp, 0, Temp.Length)
		If numb > 0 Then
			numbytes = numbytes + numb
			packets = packets + 1
			Dim bytes(numb + 1) As Byte
			BC.ArrayCopy(Temp, 0, bytes, 1, numb)
			bytes(0) = FILE_PACKET
			Streams.Write(bytes)
		End If
	Loop
	Dim bytes(1) As Byte
	bytes(0) = FILE_COPY_CLOSE
	Streams.Write(bytes)
	in.Close
	Log("Sent " & data(1) & " " & numbytes & " bytes in " & packets & " packets" )
End Sub

Sub xHandleFileDelete(buffer() As Byte)		
	Dim bytes() As Byte
	Dim data(0) As String
	Dim filename As String
	filename = BytesToString(buffer, 1, buffer.Length - 1, "UTF-8")
	data = Regex.Split(",", filename)
	Log("Deleting file " & filename)
	File.Delete(data(0), data(1))
End Sub

' /ADDED
